import argparse as _ap
import inspect as _ins

from . import argumentsDict as _argumentsDict


def _get_prefix_char(parser):
    prefix_chars = parser.prefix_chars
    try:
        return prefix_chars[0]
    except IndexError:
        return None

def by_object(obj, /, **kwargs):
    func = by_holder if hasattr(obj, "_dest") else by_func
    return func(obj, **kwargs)

def by_holder(obj, /, **kwargs):
    parents_kwargs = dict(kwargs)
    ans = _ap.ArgumentParser(
        description=obj.__doc__,
        **kwargs,
    )
    subparsers = ans.add_subparsers(dest=obj._dest, required=True)
    for n, m in _ins.getmembers(obj):
        if n.startswith("_"):
            continue
        cmd = n
        prefix_char = _get_prefix_char(ans)
        if prefix_char is not None:
            cmd = cmd.replace('_', prefix_char)
        parents_kwargs['prog'] = cmd
        parent = by_object(
            m,
            prog=cmd,
            **parents_kwargs,
        )
        subparser = subparsers.add_parser(
            cmd,
            parents=[parent],
            add_help=False,
            prog=parent.prog,
            description=parent.description,
        )
    return ans

def by_func(value, /, **kwargs):
    ans = _ap.ArgumentParser(
        description=value.__doc__,
        **kwargs,
    )
    prefix_char = _get_prefix_char(ans)
    argumentsDict = _argumentsDict.by_func(
        value, 
        prefix_char=prefix_char,
    )
    _argumentsDict.add_to_parser(
        argumentsDict,
        parser=ans,
    )
    return ans

def by_parameter(value, /, **kwargs):
    ans = _ap.ArgumentParser(
        **kwargs,
    )
    prefix_char = _get_prefix_char(ans)
    argumentsDict = _argumentsDict.by_parameter(
        value, 
        prefix_char=prefix_char,
    )
    _argumentsDict.add_to_parser(
        argumentsDict,
        parser=ans,
    )
    return ans

def by_argumentsDict(value, /, **kwargs):
    ans = _ap.ArgumentParser(**kwargs)
    _argumentsDict.add_to_parser(
        value, 
        parser=ans,
    )
    return ans
